const { cli } = require('./cli');

const PIPE = '|';
const BASHPROFILE = '~/.bash_profile';

async function gitClone(repoPath, destDir) {
  try {
    await cli('git', 'clone', `https://github.com/${repoPath}.git`, destDir);
  } catch (err) {
    console.log(`${err.message}`);
    throw err;
  }
}

async function echo(msg, redirectionPath = null) {
  try {
    await cli('echo', msg, `>> ${redirectionPath}`);
  } catch (err) {
    console.log(`${err.message}`);
    throw err;
  }
}

// $ rbenv install -l | egrep "\s[0-9]\.[0-9]\.[0-9]$" | tail -1
async function getLatestRubyVersion() {
  try {
    await cli(
      'rbenv install -l',
      PIPE,
      'grep',
      '-e',
      's[0-9].[0-9].[0-9]$',
      PIPE,
      'tail -1',
    );
  } catch (err) {
    console.log(`${err.message}`);
    throw err;
  }
}

async function installRBenv() {
  try {
    // await cli('yum update');
    await cli(
      ...[  'yum',
            'install',
            '-y',
            'git-core',
            'zlib',
            'zlib-devel',
            'gcc-c++',
            'patch',
            'readline',
            'readline-devel',
            'libyaml-devel',
            'libffi-devel',
            'openssl-devel',
            'make',
            'bzip2',
            'autoconf',
            'automake',
            'libtool',
            'bison',
            'curl',
            'sqlite-devel'
          ]
    );

    // $ git clone https://github.com/rbenv/rbenv.git ~/.rbenv
    await gitClone('rbenv/rbenv', '~/.rbenv');

    // echo 'export PATH="$HOME/.rbenv/bin:$PATH"' >> ~/.BASHPROFILE
    await echo('export PATH="$HOME/.rbenv/bin:$PATH"', BASHPROFILE);

    // echo 'eval "$(rbenv init -)"' >> ~/.BASHPROFILE
    await echo('eval "$(rbenv init -)"', BASHPROFILE);

    // source ~/.bash_profile
    // await cli('source', BASHPROFILE);

    // git clone https://github.com/rbenv/ruby-build.git ~/.rbenv/plugins/ruby-build
    await gitClone('rbenv/ruby-build', '~/.rbenv/plugins/ruby-build');
    // await cli('source', BASHPROFILE);
  } catch (err) {
    console.log(`${err.message}`);
    throw err;
  }
}

module.exports = async (version = null) => {
  let versionNumber = '2.6.1';
  try {
    await installRBenv();
    await cli('echo', "$PATH")
    console.log("Successfully installed rbenv")
    // versionNumber = version || (await getLatestRubyVersion());
    console.log(versionNumber)
  } catch (err) {
    if (err.toString().match(/enoent|not\sfound/i)) {
      await cli('rbenv', 'install', versionNumber);
      await cli('rbenv', 'global', versionNumber);

      // Env for apps
      await echo('export RACK_ENV=production', BASHPROFILE); // Generic for all Rack apps
      await echo('export RAILS_ENV="$RACK_ENV"', BASHPROFILE); // ROR
      await echo('export APP_ENV="$RACK_ENV"', BASHPROFILE); // Sinatra
      await echo('export RAILS_LOG_TO_STDOUT=true', BASHPROFILE); // ROR
    } else {
      throw err;
    }
  }
  // TODO: Add Support for JIT compilation

  return new Promise((resolve, reject) => {
    const cmd = cli('ruby', '-v')
    cmd.onerror = () => reject
    cmd.onfinish = result => console.log(result.stdout)
  });
};
